import { toast } from 'react-toastify'
import { MarkType } from '../types/drawing'
import { MarkConfigs } from '../types/mark'
import {
  decryptArrayBuffer,
  encryptArrayBuffer,
  getHash,
  isOnline,
  uploadToFileStorage
} from '../utils'
import { MarkInputSignature } from './MarkInputs/Signature'
import { MarkInputText } from './MarkInputs/Text'
import { MarkRenderSignature } from './MarkRender/Signature'
import axios from 'axios'

export const MARK_TYPE_CONFIG: MarkConfigs = {
  [MarkType.TEXT]: {
    input: MarkInputText,
    render: ({ value }) => <>{value}</>
  },
  [MarkType.SIGNATURE]: {
    input: MarkInputSignature,
    render: MarkRenderSignature,
    encryptAndUpload: async (value, encryptionKey) => {
      // Value is the stringified signature object
      // Encode it as text to the arrayBuffer
      const encoder = new TextEncoder()
      const uint8Array = encoder.encode(value)
      const hash = await getHash(uint8Array)

      if (!hash) {
        throw new Error("Can't get file hash.")
      }

      if (!encryptionKey) {
        throw new Error('Signature requires an encryption key')
      }

      // Encrypt the file contents with the same encryption key from the create signature
      const encryptedArrayBuffer = await encryptArrayBuffer(
        uint8Array,
        encryptionKey
      )

      // Create the encrypted json file from array buffer and hash
      const file = new File([encryptedArrayBuffer], `${hash}.json`)

      if (await isOnline()) {
        try {
          const url = await uploadToFileStorage(file)
          toast.success('files.zip uploaded to file storage')
          return url
        } catch (error) {
          if (error instanceof Error) {
            toast.error(error.message || 'Error occurred in uploading file')
          }
        }
      } else {
        // Handle offline?
      }

      return value
    },
    fetchAndDecrypt: async (value, encryptionKey) => {
      if (!encryptionKey) {
        throw new Error('Signature requires an encryption key')
      }

      const encryptedArrayBuffer = await axios.get(value, {
        responseType: 'arraybuffer'
      })

      const arrayBuffer = await decryptArrayBuffer(
        encryptedArrayBuffer.data,
        encryptionKey
      ).catch((err) => {
        console.log('err in decryption:>> ', err)
        return null
      })

      if (arrayBuffer) {
        // decode json
        const decoder = new TextDecoder()
        const value = decoder.decode(arrayBuffer)
        return value
      }

      // Handle offline?
      return value
    }
  }
}
